#!/usr/bin/env python3

"""This script reads the eddy graph from Matlab v6 files
    id_child_(anti|cyclo).mat and N(anti|cyclo).mat, and writes the
    graph to a new file in edgelist format.

"""

import scipy.io as sio
import csv
import sys
import json
import convert_id

def adjust_n(n_Matlab):
    k_Matlab = (n_Matlab - 1) // e_overestim
    k = date_num[k_Matlab] - date_num[0]
    return n_Matlab + (k - k_Matlab) * e_overestim

orientation = sys.argv[1] # "cyclo" or "anti"
id_child = sio.loadmat(f"id_child_{orientation}.mat", squeeze_me=True)\
    ["id_child"]
n_eddies = sio.loadmat(f"N{orientation}.mat", squeeze_me = True)\
    [f"N{orientation}"].astype(int, casting = "safe", copy = False)
date_num = sio.loadmat("date_num.mat", squeeze_me = True)["date_num"]
n_dates = n_eddies.size
print("n_dates =", n_dates)
e_overestim = id_child.shape[0]
print("e_overestim =", e_overestim)
d_init = convert_id.days_1950(date_num[0])

with open("node_id_param.json", "w") as f:
    json.dump({"e_overestim": e_overestim, "d_init": d_init}, f)

with open(f"edgelist_{orientation}.csv", "w", newline = '') as edgelist:
    writer = csv.writer(edgelist, delimiter = ' ', lineterminator = "\n")

    for j1 in range(n_dates):
        for i1, id_child_1 in enumerate(id_child[:n_eddies[j1], j1]):
            n1 = (date_num[j1] - date_num[0]) * e_overestim + i1 + 1
            # id_child_1 may be an int, which means there is a single
            # successor, or id_child_1 may be a numpy array, which means
            # there are several successors.

            try:
                # Try processing id_child_1 as an array:
                for n_Matlab in id_child_1:
                    n2 = adjust_n(n_Matlab)
                    writer.writerow((n1, n2))
            except TypeError:
                # id_child_1 is a scalar int.
                n2 = adjust_n(id_child_1)
                writer.writerow((n1, n2))
