#!/usr/bin/env python3

import netCDF4
from mpl_toolkits import basemap
import jumble
from matplotlib import pyplot as plt
import glob
import itertools

m = basemap.Basemap(llcrnrlon=-30, llcrnrlat=-90, urcrnrlon=60, urcrnrlat = 0)
plt.figure()
m.drawcoastlines()
m.drawmeridians(range(0,361, 15), labels = [0, 0, 0, 1])
m.drawparallels(range(-90,91, 15), labels = [1, 0, 0, 0])
colors = itertools.cycle(['blue', 'orange', 'green', 'red', 'purple', 'brown',
                          'pink', 'gray', 'olive', 'cyan'])

for filename in glob.glob("h_region_*.nc") + ["h_outermost.nc"]:
    with netCDF4.Dataset(filename) as f:
        if "lon" in f.variables:
            lon = "lon"
            lat = "lat"
        else:
            lon = "longitude"
            lat = "latitude"
            
        jumble.draw_bbox(f[lon][0], f[lon][-1], f[lat][0], f[lat][-1],
                         colors = next(colors), label = filename)

plt.legend(loc = "lower left", bbox_to_anchor = (1, 0.5))
plt.subplots_adjust(right = 0.7)
plt.savefig("regions.pdf")
##plt.show()
