#!/usr/bin/env python3

import argparse
import datetime
import shutil
import nco
import os
from os import path
import subprocess
import shapefile
import sys
from dateutil import parser

argparser = argparse.ArgumentParser()
argparser.add_argument("first_file",
                       help = "NetCDF file containing SSH and velocity at a "
                       "single date")
argparser.add_argument("date_format",
                       help = "template of basename with date format")
argparser.add_argument("-l", "--last_date", help = "%%Y-%%m-%%d")
argparser.add_argument("-b", "--bbox", nargs=4, type = float,
                       metavar = ("xmin", "xmax", "ymin", "ymax"))
args = argparser.parse_args()

if not os.access("main_nml.txt", os.R_OK):
    sys.exit("main_nml.txt not found or not readable")

inst_eddies_exe = "@CMAKE_CURRENT_BINARY_DIR@/inst_eddies"

if not os.access(inst_eddies_exe, os.X_OK):
    sys.exit(inst_eddies_exe + " not found or not executable")

for orient in ["cyclo", "anti"]:
    if os.access(f"SHPC_{orient}_all_dates", os.F_OK):
        shutil.rmtree("SHPC_all_dates")
        
dirname, basename = path.split(args.first_file)
my_date = datetime.datetime.strptime(basename, args.date_format).date()
nc_file = args.first_file
nco_instance = nco.Nco()

if args.last_date:
    last_date = parser.parse(args.last_date).date()
else:
    last_date = my_date

while True:
    print("inst_eddies.py: Date:", my_date)
    
    if os.access(nc_file, os.F_OK):
        if args.bbox:
            xmin, xmax, ymin, ymax = args.bbox
            nco_instance.ncks(nc_file, output = "cropped.nc",
                              options = [f"--dimension=longitude,{xmin},{xmax}",
                                         f"--dimension=latitude,{ymin},{ymax}"])
            nc_file = "cropped.nc"
            
        nco_instance.ncpdq(nc_file, output = "unpacked.nc",
                           options = ["--unpack"])
        nc_file = "unpacked.nc"
        os.symlink(nc_file, "h.nc")
        os.symlink(nc_file, "uv.nc")
        os.mkdir("SHPC_cyclo")
        os.mkdir("SHPC_anti")
        
        with open("main_nml.txt") as nml_file:
            subprocess.run(inst_eddies_exe, stdin = nml_file, check = True)

        os.remove("h.nc")
        os.remove("uv.nc")
    else:
        print("inst_eddies.py: Missing file:", nc_file)

    for orient in ["cyclo", "anti"]:
        if os.access(f"SHPC_{orient}_all_dates", os.F_OK):
            if os.access(f"SHPC_{orient}", os.F_OK):
                for filename in ["extremum", "max_speed_contour",
                                 "outermost_contour"]:
                    from_file = path.join(f"SHPC_{orient}", filename)
                    to_file = path.join(f"SHPC_{orient}_all_dates", filename)

                    for command in ["dbfcat", "shpcat"]:
                        subprocess.run([command, from_file, to_file],
                                       check = True)

                shutil.rmtree(f"SHPC_{orient}")

            with open(f"SHPC_{orient}_all_dates/ishape_last.txt", "a") as \
                 ishape_last, \
                 shapefile.Reader(f"SHPC_{orient}_all_dates/extremum") as shpr:
                n_shapes = len(shpr)
                ishape_last.write(str(n_shapes - 1) + "\n")
        else:
            if os.access(f"SHPC_{orient}", os.F_OK):
                os.rename(f"SHPC_{orient}", f"SHPC_{orient}_all_dates")

    my_date += datetime.timedelta(1)
    if my_date > last_date: break
    basename = my_date.strftime(args.date_format)
    nc_file = path.join(dirname, basename)

os.remove("unpacked.nc")
if args.bbox: os.remove("cropped.nc")
