module write_overlap_m

  use derived_types, only: eddy

  implicit none
  
  private eddy
  type(eddy), private, save:: e

contains

  subroutine write_overlap(hshp_interp, k, delta, i1, i2, m, i_interp, &
       cyclone, coord_extr_1, ssh_extr_1, coord_extr_2, ssh_extr_2, w)

    ! Writes edges to unit_edge_(cyclo|anti) and interpolated eddies
    ! to shapefiles, between (k - delta, i1) and (k, i2).

    ! Libraries:
    use nr_util, only: twopi

    use derived_types, only: shp_tr
    use write_eddy_m, only: write_eddy
    use write_edge_m, only: write_edge

    TYPE(shp_tr), intent(in):: hshp_interp
    integer, intent(in):: k ! date index
    integer, intent(in):: delta, i1, i2, m
    integer, intent(in):: i_interp(:) ! (delta - 1)
    logical, intent(in):: cyclone
    real, intent(in):: coord_extr_1(:) ! (2)
    real, intent(in):: ssh_extr_1

    real, intent(in):: coord_extr_2(:) ! (2)
    ! (Note: value attribute not allowed for an array.)

    real, intent(in):: ssh_extr_2
    real, intent(in):: w

    ! Local:
    integer i_pred, j_interp, k_interp
    real w_interp
    real coord_extr_2_local(2)

    !-------------------------------------------------------------------

    if (delta >= 2) then
       ! Prepare interpolation:

       e%cyclone = cyclone

       ! Define coord_extr_2_local with the longitude of extremum i2 shifted
       ! to a value close to the longitude of extremum i1:
       coord_extr_2_local(1) = coord_extr_2(1) + m * twopi
       coord_extr_2_local(2) = coord_extr_2(2)
    end if

    i_pred = i1

    do j_interp = 1, delta - 1
       ! Interpolate extremum:
       w_interp = j_interp / real(delta)
       e%coord_extr = (1 - w_interp) * coord_extr_1 &
            + w_interp * coord_extr_2_local
       e%ssh_extr = (1 - w_interp) * ssh_extr_1 + w_interp * ssh_extr_2

       k_interp = k - delta + j_interp
       call write_eddy(e, hshp_interp, k_interp, i_interp(j_interp))
       call write_edge(cyclone, k_interp - 1, i_pred, k_interp, &
            i_interp(j_interp), w)
       i_pred = i_interp(j_interp)
    end do

    call write_edge(cyclone, k - 1, i_pred, k, i2, w)

  end subroutine write_overlap

  !**************************************************************************

  subroutine init_interpolated_eddy

    ! Set some null components of the interpolated eddy:

    use derived_types, only: null_ssh_contour, missing_speed

    !-----------------------------------------------------------------------
    
    e%out_cont = null_ssh_contour()
    e%speed_cont = null_ssh_contour()
    e%max_speed = missing_speed
    e%valid = .false.
    e%interpolated = .true.
    e%radius4 = 0

  end subroutine init_interpolated_eddy

end module write_overlap_m
